#ifndef _FASVAR_H_
#define _FASVAR_H_

#define FAS_NEED_VM_PATCH	1

#define MAXCHAIN	(MAXPHYS/NBPG+1)
#define MAXPENDING	9	/* 7 IDs + 2 extra */

struct	fas_dma_chain {
	vm_offset_t	ptr;
	u_short		len;
	short		flg;
};
#define FAS_CHAIN_DMA	0x00
#define FAS_CHAIN_BUMP	0x01
#define FAS_CHAIN_PRG	0x02

#ifdef FAS_NEED_VM_PATCH
struct	vm_link_data {
	vm_offset_t	pa[MAXCHAIN];
	short		offset;
	short		pages;
};
#endif

struct	fas_pending {
	TAILQ_ENTRY(fas_pending) link;
	struct scsi_xfer	*xs;
#ifdef FAS_NEED_VM_PATCH
	struct vm_link_data	 vm_link_data;
#endif
};

struct nexus {
	struct	scsi_xfer 	*xs;

	u_char			 cbuf[14];
	short			 clen;

#ifdef FAS_NEED_VM_PATCH
	struct vm_link_data	 vm_link_data;
#endif
	struct fas_dma_chain	 dma[MAXCHAIN];
	short			 max_link;
	short			 cur_link;

	u_char			*buf;		/* va */
	int			 len;

	vm_offset_t		 dma_buf;	/* pa */
	int			 dma_len;

	vm_offset_t		 dma_blk_ptr;
	int			 dma_blk_len;
	u_char			 dma_blk_flg;

	u_char			 state;
	u_short			 flags;

	short			 period;
	u_char			 offset;

	u_char			 syncper;
	u_char			 syncoff;
	u_char			 config3;

	u_char			 lun_unit;
	u_char			 status;
};

/* SCSI nexus_state */
#define FAS_NS_IDLE		0
/* ---------------- */
#define FAS_NS_SELECTED		1
#define FAS_NS_DATA_IN		2
#define FAS_NS_DATA_OUT		3
#define FAS_NS_STATUS		4
#define FAS_NS_MSG_IN		5
#define FAS_NS_MSG_OUT		6
#define FAS_NS_SVC		7
#define FAS_NS_DISCONNECTED	8
#define FAS_NS_DONE		9
#define FAS_NS_FINISHED		10
#define FAS_NS_SENSE		11

/* SCSI nexus flags */
#define FAS_UNIT_BUSY		0x001
#define FAS_HAS_MSG		0x002
#define FAS_REQUEST_SENSE	0x004
#define FAS_SENSING		0x008
#define FAS_GOT_DISCONN		0x010
#define FAS_DISCONNECTED	0x020
#define FAS_DO_SDTR		0x040
#define FAS_SDTR_SENT		0x080
#define FAS_SYNC_TESTED		0x100
#define FAS_IMMEDIATE		0x200
#define FAS_DEBUG		0x800

struct	fas_softc {
/* System struct */
	struct	device		 sc_dev;
	struct	scsi_link	 sc_link;	/* proto for sub devices */

	TAILQ_HEAD(,fas_pending) sc_xs_pending;
	TAILQ_HEAD(,fas_pending) sc_xs_free;
	struct	fas_pending 	 sc_xs_store[MAXPENDING];

	fas_regmap_p		 sc_fas;	/* FAS216 Address */
	void			*sc_spec;	/* Board-specific data */

#ifdef FAS_NEED_VM_PATCH
	u_char			*sc_vm_link;
#endif
	
	u_char			*sc_bump_va;	/* Bumpbuf virtual adr */
	vm_offset_t		 sc_bump_pa;	/* Bumpbuf physical adr */
	int			 sc_bump_sz;	/* Bumpbuf size */

/* Configuration registers, must be set BEFORE fasinitialize */
	u_char			 sc_clock_freq;
	u_short			 sc_timeout;
	u_char			 sc_host_id;
	u_char			 sc_config_flags;

/* Generic DMA functions */
	int		       (*sc_setup_dma)();
	int		       (*sc_build_dma_chain)();
	int		       (*sc_need_bump)();

/* Generic Led data */
	int			 sc_led_status;
	void		       (*sc_led_on)();
	void		       (*sc_led_off)();

/* Nexus list */
	struct nexus		 sc_nexus[8];
	struct nexus		*sc_cur_nexus;

/* Current transfer data */
#ifdef FAS_NEED_VM_PATCH
	struct vm_link_data	 sc_vm_link_data;
#endif

	u_char			*sc_buf;	/* va */
	int			 sc_len;

	vm_offset_t		 sc_dma_buf;	/* pa */
	int			 sc_dma_len;
	vm_offset_t		 sc_dma_blk_ptr;
	int			 sc_dma_blk_len;
	short			 sc_dma_blk_flg;

	struct fas_dma_chain	*sc_chain;	/* Current DMA chain */
	short			 sc_max_link;
	short			 sc_cur_link;

/* Interrupt registers */
	u_char			 sc_status;
	u_char			 sc_interrupt;
	u_char			 sc_resel[2];

	u_char			 sc_units_disconnected;

/* Storage for FAS216 config registers (current values) */
	u_char			 sc_config1;
	u_char			 sc_config2;
	u_char			 sc_config3;
	u_char			 sc_clock_conv_fact;
	u_char			 sc_timeout_val;
	u_char			 sc_clock_period;

	u_char			 sc_msg_in[7];
	u_char			 sc_msg_in_len;

	u_char			 sc_msg_out[7];
	u_char			 sc_msg_out_len;

	u_char			 sc_unit;
	u_char			 sc_lun;
	u_char			 sc_flags;
};

#define FAS_DMA_READ		0
#define FAS_DMA_WRITE		1
#define FAS_DMA_CLEAR		2

/* sc_flags */
#define FAS_RESEL_ENABLED	0x01
#define FAS_ACTIVE		0x02
#define FAS_DONT_WAIT		0x04
#ifdef FAS_NEED_VM_PATCH
#define FAS_HAS_VM_LINK		0x08
#endif

/* SCSI Selection modes */
#define FAS_SELECT	0x00	/* Normal selection: No sync, no resel */
#define FAS_SELECT_R	0x01	/* Reselection allowed */
#define FAS_SELECT_S	0x02	/* Synchronous transfer allowed */
#define FAS_SELECT_I	0x04	/* Selection for fasicmd */
/* Nice abbreviations of the above */
#define FAS_SELECT_RS	(FAS_SELECT_R|FAS_SELECT_S)
#define FAS_SELECT_RI	(FAS_SELECT_R|FAS_SELECT_I)
#define FAS_SELECT_SI	(FAS_SELECT_S|FAS_SELECT_I)
#define FAS_SELECT_RSI	(FAS_SELECT_R|FAS_SELECT_S|FAS_SELECT_I)

/* sc_config_flags */
#define FAS_NO_SYNCH	0x01	/* Disable synchronous transfer */
#define FAS_SLOW_CABLE	0x02	/* Cable is "unsafe" for fast scsi-2 */
#define FAS_NO_DMA	0x04	/* Do not use DMA! EVER! */
#define FAS_NO_RESELECT	0x08	/* Do not allow relesection */

void	fasinitialize __P((struct fas_softc *sc));
void	fas_minphys __P((struct buf *bp));
#if !NEW_CODE
u_int	fas_adinfo __P((void));
#endif
int	fas_scsicmd __P((struct scsi_xfer *));

#endif /* _FASVAR_H_ */
